/* The FreeDOS-32 FAT Driver version 2.0
 * Copyright (C) 2001-2005  Salvatore ISAJA
 *
 * This file "dir.c" is part of the FreeDOS-32 FAT Driver (the Program).
 *
 * The Program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * The Program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the Program; see the file GPL.txt; if not, write to
 * the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/**
 * \file
 * \brief Facilities to access directories.
 */
/**
 * \addtogroup fat
 * @{
 */
#include "fat.h"


/*****************************************************************************
 *
 * Facilities to manage short file names
 *
 *****************************************************************************/

static const char *invalid_sfn_characters = "\"+,./:;<=>[\\]|*?";

/**
 * \brief Expands a file name in FCB format to a wide character string.
 * \param nls       NLS operations to use for character set conversion;
 * \param dest      array to hold the converted name, not null terminated;
 * \param dest_size max number of wide characters to write in \c dest;
 * \param source    11-bytes array holding the name in FCB format to convert.
 * \return The length in wide characters of the expanded name, or a negative error.
 */
//static int expand_fcb_name(const struct nls_operations *nls, wchar_t *dest, size_t dest_size, const uint8_t *source)
static int expand_fcb_name(char *dest, size_t dest_size, const uint8_t *source)
{
	const uint8_t *name_end, *ext_end;
	const uint8_t *s = source;
	char aux[13], *a = aux;
	int res;

	/* Skip padding spaces at the end of the name and the extension */
	for (name_end = source + 7; *name_end == ' '; name_end--)
		if (name_end == source) return -EINVAL;
	for (ext_end = source + 10; *ext_end == ' '; ext_end--)
		if (ext_end == source) return -EINVAL;

	/* Copy name dot extension in aux */
	if (*s == 0x05) *a++ = (char) FAT_FREEENT, s++;
	for (; s <= name_end; *a++ = (char) *s++);
	if (source + 8 <= ext_end) *a++ = '.';
	for (s = source + 8; s <= ext_end; *a++ = (char) *s++);
	*a = 0;

	/* Convert aux to a wide character string */
	for (a = aux, res = 0; *a; dest++, dest_size--, res++)
	{
		int skip;
		if (!dest_size) return -ENAMETOOLONG;
//		skip = nls->mbtowc(dest, a, aux + sizeof(aux) - a);
		//added manually
		*dest = (unsigned char) *a;
		skip = 1;
		//end manual add
		if (skip < 0) return skip;
		a += skip;
	}
	return res;
}

/*****************************************************************************
 *
 * Facilities to read directory entries
 *
 *****************************************************************************/


/**
 * \brief   Computes the location of a directory entry.
 * \param   c   the directory containing the directory entry to locate;
 * \param   lud LookupData structure to update with the directory entry location.
 * \remarks It is assumed that this function is called right after reading or
 *          writing the directory entry. Updates the \c de_dirofs,
 *          \c de_sector and \c de_secofs fields of \c lud.
 */
static void direntry_location(const Channel *c, LookupData *lud)
{
	const File   *f = c->f;
	const Volume *v = f->v;
	lud->de_dirofs = c->file_pointer - sizeof(struct fat_direntry);
	lud->de_sector = lud->de_dirofs >> v->log_bytes_per_sector;
	if (!f->de_sector && !f->first_cluster)
		lud->de_sector += v->root_sector;
	else
		lud->de_sector = (lud->de_sector & (v->sectors_per_cluster - 1))
	                       + ((c->cluster - 2) << v->log_sectors_per_cluster) + v->data_start;
	lud->de_secofs = lud->de_dirofs & (v->bytes_per_sector - 1);
}


/**
 * \brief  Backend for the readdir and find services.
 * \param  c   the open instance of the directory to read;
 * \param  lud LookupData structure to fill with the read data.
 * \return 0 on success, or a negative error.
 */
int fat_do_readdir(Channel *c, LookupData *lud)
{
	File   *f = c->f;
	Volume *v = f->v;
	if (!(f->de.attr & FAT_ADIR)) return -EBADF;
	for (;;)
	{
		struct fat_direntry *de = &lud->cde;
		int num_read = fat_read(c, de, sizeof(struct fat_direntry));
		if (num_read < 0) return num_read;
		if (!num_read) break; /* EOF */
		if (num_read != sizeof(struct fat_direntry)) return -EFTYPE; /* malformed directory */
		if (de->name[0] == FAT_ENDOFDIR) break;
		if ((de->name[0] != FAT_FREEENT) && (!(de->attr & FAT_AVOLID) || (de->attr == FAT_AVOLID)))
		{
			int res;
//			res = expand_fcb_name(v->nls, lud->sfn, FAT_SFN_MAX, de->name);
			res = expand_fcb_name(lud->sfn, FAT_SFN_MAX, de->name);
			if (res < 0) return res;
			lud->sfn_length = res;
			direntry_location(c, lud);
			return 0;
		}
		
	}
	return -ENMFILE;
}

/* @} */
